//______________________________________________________________________________
//
// Copyright (c) Cypress Semiconductor, 2003
// All rights reserved.
//
// =============================================================================
// ALERT.  ALERT.  Advanced Developer Example Code
//
// This example code demonstrates one of the MOST COMPLEX and DIFFICULT ways to
// utilize the CYAPI.lib.  
//
// The technique demonstrated in this example is only needed in very special 
// circumstances and is intended for expert developers who already have mastered
// the fundamentals of the CYAPI object model.  
//
// It is strongly advised that, instead of starting with this example, you begin 
// by using the XferData() method demonstrated in the BulkLoop example code.
//  
// That is, you are advised to only use this example code if directed to do so by
// Cypress technical support.
//
// In almost every case, problems that you might encounter can more easily be 
// debugged and duplicated using the XferData() method, rather than the 
// [BeginDataXfer - WaitForXfer - FinishDataXfer] (aka BWF) approach shown in 
// this example.
//
// When submitting USB Developer Studio technical problems to Cypress, please 
// demonstrate the problem with application code that uses the XferData() method 
// and not the BWF approach.
// =============================================================================
//
// This application demonstrates use of the CyAPI asynchronous IO methods,
// BeginDataXfer, WaitForXfer, and FinishDataXfer.  Use of these methods allows
// multiple data transfer requests to be queued to the device before beginning to
// service the returned data.  
//
// The intent is to pre-load a queue of requests such that the device always has 
// an outstanding request to satisfy.  If this objective is achieved, the USBus 
// should stay fully utilized and create a continous "Stream" of data.
//
// This application is intended to be used with a Cypress FX2 device running the 
// CYStream.iic firmware.
//
//______________________________________________________________________________

#include "stdafx.h"
#include "Streamer.h"
#include "StreamerDlg.h"
#include <math.h>
#include <memory.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define STATUS_POLL_TIMER_MS			500    // pull status every 1/2 sec

static LPCTSTR gszProcessorTime=TEXT("\\Processor(_Total)\\% Processor Time");


/////////////////////////////////////////////////////////////////////////////
// CStreamerDlg dialog

CStreamerDlg::CStreamerDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CStreamerDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CStreamerDlg)
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDI_ICON2);
}

void CStreamerDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CStreamerDlg)
	DDX_Control(pDX, IDC_DEVICENAME, m_DeviceName);
	DDX_Control(pDX, IDC_DRIVERVER, m_DriverVer);
	DDX_Control(pDX, IDC_AVGCPUVALUE, m_AvgCpuValue);
	DDX_Control(pDX, IDC_CPUVALUE, m_CpuValue);
	DDX_Control(pDX, IDC_CPUPROGRESS, m_CpuProgress);
	DDX_Control(pDX, IDC_PACKETSCOMBO, m_PacketsCombo);
	DDX_Control(pDX, IDC_BULKRADIO, m_BulkRadBtn);
	DDX_Control(pDX, IDC_ISOCRADIO, m_IsocRadBtn);
	DDX_Control(pDX, IDC_RATE, m_Rate);
	DDX_Control(pDX, IDC_XferSize, m_XferSzLabel);
	DDX_Control(pDX, IDOK, m_StartStop);
	DDX_Control(pDX, IDC_TIMEOUT, m_TimeOut);
	DDX_Control(pDX, IDC_FAILURE_CNT, m_FailureLabel);
	DDX_Control(pDX, IDC_SUCCESS_CNT, m_SuccessLabel);
	DDX_Control(pDX, IDC_EDIT1, m_QueueSzEdit);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CStreamerDlg, CDialog)
	//{{AFX_MSG_MAP(CStreamerDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_ISOCRADIO, OnIsocradio)
	ON_BN_CLICKED(IDC_BULKRADIO, OnBulkradio)
	ON_WM_TIMER()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CStreamerDlg message handlers

BOOL CStreamerDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	Successes		= 0;
	Failures		= 0;
	BytesXferred	= 0;
	Rate			= 0;


	USBDevice = new CCyUSBDevice();

	if (USBDevice->DeviceCount())
	{
		char szBuf[32];

		wsprintf ( szBuf, "0x%08X", USBDevice->DriverVersion);
		m_DriverVer.SetWindowText(szBuf);

		m_DeviceName.SetWindowText(USBDevice->DeviceName);

		if ((USBDevice->ProductID == 0x1003) && (USBDevice->ProductID == 0x1003)) // CyStream Device 
		{
			USBDevice->SetAltIntfc(3);  // Has Isoc In EndPt
		}
	}
	else
	{
		m_DriverVer.SetWindowText("");
		m_DeviceName.SetWindowText("");
	}


	sLastPPX = "16";
	m_QueueSzEdit.SetWindowText("8");
	m_TimeOut.SetWindowText("1500");
	m_PacketsCombo.SetCurSel(1);
	
	bStreaming = false;

	if (USBDevice->IsocInEndPt)
	  OnIsocradio();
	else
	  OnBulkradio();


	XferThread = NULL;

	// Initialization for CPU Performance
	hQuery = NULL;
	hCounter = NULL;
	bCounterInit = false;
	ulCpuCycles = 0;
	ulCpuCount = 0;

	if (PdhOpenQuery ( NULL, 1, &hQuery ) == ERROR_SUCCESS)
	{
		if (PdhAddCounter ( hQuery, gszProcessorTime, 
			NULL, &hCounter) == ERROR_SUCCESS)
		{
			bCounterInit = TRUE;
		}
	}

	  
    // Start timer
    m_nTimer = SetTimer(1, STATUS_POLL_TIMER_MS, 0);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CStreamerDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CStreamerDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}



/////////////////////////////////////////////////////////////////////////////
// Get CPU Cycle function
//
// Obtains the raw CPU counter value, formats the CPU counter and returns 
// the value.

LONG CStreamerDlg::GetCPUCycle()
{
	PDH_RAW_COUNTER	ppdhRawCounter;
	PDH_FMT_COUNTERVALUE pdhFormattedValue;

	// Collect the current raw data value for all counters in the 
	// specified query and updates the status code of each counter 
	if (PdhCollectQueryData ( hQuery ) != ERROR_SUCCESS)
	{
		// Data collection failed.
		return 0;
	}

	// Get the CPU raw counter value	
	if (PdhGetRawCounterValue ( hCounter, NULL, &ppdhRawCounter ) == ERROR_SUCCESS)
	{
		// Format the CPU counter
		if (PdhGetFormattedCounterValue ( hCounter, PDH_FMT_LONG, NULL, 
			&pdhFormattedValue) == ERROR_SUCCESS)
		{
			// Got the formatted value.  Show it on the graph
			return( pdhFormattedValue.longValue );
		}
	}

	return 0;
}

void *memmem(void *haystack, int hlen, void *needle, int nlen)
{
	for (int o = 0; o < hlen - nlen; ++o)
		if (!memcmp(((unsigned char*)haystack) + o, needle, nlen))
			return ((unsigned char*)haystack) + o;
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// The looping Xfer Thread function
//
// The looping xfers need to run in a separate thread so as to be stoppable 
// via a click on the "Stop" button.  Otherwise, that click will never get 
// serviced and the app will loop indefinitely.
//
// The "Start" button on click handler just calls this routine, passing a pointer
// to the dialog (main window) object.  That way, this routine has access to all
// the public members of the CStreamerDlg class.

UINT XferLoop( LPVOID params ) {

    CStreamerDlg *dlg = (CStreamerDlg *) params;
	char s[24];	ZeroMemory(s, 24);

	// UI Stuff
	dlg->m_QueueSzEdit.GetLine(0,s,5);
	int QueueSize  = atoi(s);
	if (QueueSize < 1) QueueSize = 1;
	
	dlg->m_TimeOut.GetLine(0,s,5);
	int Timeout  = atoi(s);
	if (Timeout < 1) Timeout = 1;

	int n = dlg->m_PacketsCombo.GetCurSel();
	int PPX = (int) pow(2,dlg->m_PacketsCombo.GetCurSel() + 3);
	if (PPX < 0) PPX = 1;

	if (!dlg->USBDevice->IsOpen()) {
 	  AfxMessageBox("No USB device found.", MB_OK | MB_ICONEXCLAMATION);
	  dlg->bStreaming = false;
      dlg->m_StartStop.SetWindowText("Start");
	  dlg->XferThread = NULL;
      dlg->ulCpuCycles = 0;
      dlg->ulCpuCount = 0;
	  return false;
	}


	if (!dlg->InEndPt) {
 	  AfxMessageBox("No IN endpoint found.", MB_OK | MB_ICONEXCLAMATION);
	  dlg->bStreaming = false;
      dlg->m_StartStop.SetWindowText("Start");
	  dlg->XferThread = NULL;
      dlg->ulCpuCycles = 0;
      dlg->ulCpuCount = 0;
	  return false;
	}

	dlg->m_SuccessLabel.SetWindowText("0");
	dlg->m_FailureLabel.SetWindowText("0");
	dlg->Successes		= 0;
	dlg->Failures		= 0;
	dlg->BytesXferred	= 0;

	// Allocate the arrays needed for queueing
	PUCHAR     *buffers = new PUCHAR[QueueSize];
	CCyIsoPktInfo **isoPktInfos = new CCyIsoPktInfo*[QueueSize];
	PUCHAR    *contexts = new PUCHAR[QueueSize];
	OVERLAPPED *inOvLap = new OVERLAPPED[QueueSize];

	LONG i;
	LONG len = dlg->InEndPt->MaxPktSize * PPX; // Each xfer request will get PPX isoc packets

	dlg->InEndPt->SetXferSize(len);

	for (i=0; i< QueueSize; i++) { 
       buffers[i]        = new UCHAR[len];
       isoPktInfos[i]    = new CCyIsoPktInfo[PPX];
	   memset(&(inOvLap[i]),0,sizeof(OVERLAPPED));
       inOvLap[i].hEvent = CreateEvent(NULL, false, false, NULL);
	}

	// Queue-up the first batch of transfer requests
	for (i=0; i< QueueSize; i++) 
	   contexts[i] = dlg->InEndPt->BeginDataXfer(buffers[i], len, &inOvLap[i]);

		
		
    i=0;

	FILE *f = fopen("dump", "wb");

	dlg->t1 = GetTickCount();

	for (;dlg->bStreaming;) {
		LONG rLen = len;  // Reset this each time through because
		                  // FinishDataXfer may modify it

		// Wait for the xfer to complete. 
		if (!dlg->InEndPt->WaitForXfer(&inOvLap[i], Timeout)) { 
			dlg->InEndPt->Abort();
            // Wait for the stalled command to complete 
            WaitForSingleObject(inOvLap[i].hEvent,INFINITE);
		}

		// Must always call FinishDataXfer to release memory of contexts[i]
		if (dlg->bIsoc){

			if (dlg->InEndPt->FinishDataXfer(buffers[i], rLen, &inOvLap[i], contexts[i], isoPktInfos[i])) {

		
				dlg->t2 = GetTickCount();

	  			CCyIsoPktInfo *pkts = isoPktInfos[i];
				for (int j=0; j< PPX; j++) {

					if (pkts[j].Status)
						dlg->Failures++;
					else {
						dlg->Successes++;
						dlg->BytesXferred += pkts[j].Length;
					}

					pkts[j].Length = 0; // Reset to zero for re-use.

				}
			} else
				dlg->Failures++; 
			
		} else { 

			if (dlg->InEndPt->FinishDataXfer(buffers[i], rLen, &inOvLap[i], contexts[i])) {			
				ASSERT(!(rLen&1));

				for (int a=0; a<rLen/2; ++a)
				{
					int v = (buffers[i][a*2] & 0xF0) | ((buffers[i][a*2+1]>>4)&0xF);
					buffers[i][a] = v;
				}

				fwrite(buffers[i], rLen/2, 1, f);

				dlg->t2 = GetTickCount();
				dlg->Successes++;
				dlg->BytesXferred += rLen;
			} else
				dlg->Failures++; 
		}


		DWORD deltaT = (dlg->t2 - dlg->t1);
		if (deltaT > 0) 
			dlg->Rate = ((dlg->BytesXferred / deltaT) * 1000) / 1024; // Convert to KB
		
		// Re-submit this request to keep the queue full
	    contexts[i] = dlg->InEndPt->BeginDataXfer(buffers[i], len, &inOvLap[i]);

		i++;
		if (i == QueueSize) i=0;

	}
	fclose(f);


	// Clean-up
	dlg->InEndPt->Abort();
	for (i=0; i< QueueSize; i++) { // Wait for all the queued requests to be cancelled 
	  LONG rLen = len;



	  if (!dlg->InEndPt->WaitForXfer(&inOvLap[i], Timeout)) { 

		dlg->InEndPt->Abort();

        // Wait for the stalled command to complete 
        WaitForSingleObject(inOvLap[i].hEvent,INFINITE);
	  }


	  // Release contexts[i]
	  if (dlg->bIsoc)
	    dlg->InEndPt->FinishDataXfer(buffers[i], rLen, &inOvLap[i], contexts[i], isoPktInfos[i]);
	  else 
	    dlg->InEndPt->FinishDataXfer(buffers[i], rLen, &inOvLap[i], contexts[i]);

	  CloseHandle(inOvLap[i].hEvent);

	  delete [] buffers[i];

	  delete [] isoPktInfos[i];
	}

	delete [] buffers;
	delete [] isoPktInfos;
	delete [] contexts;
	delete [] inOvLap;

	dlg->XferThread = NULL;
	return true;
}



void CStreamerDlg::OnOK() 
{

    if (XferThread) {
		bStreaming = false;
		m_StartStop.SetWindowText("Start");
		XferThread = NULL;
 		m_IsocRadBtn.EnableWindow(true);
		m_BulkRadBtn.EnableWindow(true);
		ulCpuCycles = 0;
		ulCpuCount = 0;
	} else {

		bStreaming = true;
		m_StartStop.SetWindowText("Stop");
		m_IsocRadBtn.EnableWindow(false);
		m_BulkRadBtn.EnableWindow(false);

		XferThread = AfxBeginThread(XferLoop, this);

		Successes		= 0;
		Failures		= 0;
		BytesXferred	= 0;
		Rate			= 0;

		ulCpuCycles = 0;
		ulCpuCount = 0;
	}


}


void CStreamerDlg::OnClose() 
{
    if (XferThread) {
	  bStreaming = false;
	  MessageBox("Streaming stopped.");
	}

	delete USBDevice;
	
	CDialog::OnClose();
}

void CStreamerDlg::OnIsocradio() 
{

	bIsoc = true;
	USBDevice->Open(0);
	USBDevice->SetAltIntfc(3);  // If CYStream device, has Isoc In EndPt
	InEndPt = USBDevice->IsocInEndPt;

	m_IsocRadBtn.SetCheck(BST_CHECKED);
	m_BulkRadBtn.SetCheck(BST_UNCHECKED);

	if (InEndPt) {
	  CString xferSzStr;
	  xferSzStr.Format("%d",InEndPt->MaxPktSize);
	  m_XferSzLabel.SetWindowText(xferSzStr);
	  m_PacketsCombo.SetCurSel(1);
	  m_QueueSzEdit.SetWindowText("8");
    } else
	  m_XferSzLabel.SetWindowText("0");

}

void CStreamerDlg::OnBulkradio() 
{

	bIsoc = false;
	USBDevice->Open(0);
	USBDevice->SetAltIntfc(0);  // If CYStream device, has Bulk In EndPt
	InEndPt = USBDevice->BulkInEndPt;

	m_IsocRadBtn.SetCheck(BST_UNCHECKED);
	m_BulkRadBtn.SetCheck(BST_CHECKED);

	if (InEndPt) {
	  CString xferSzStr;
	  xferSzStr.Format("%d",InEndPt->MaxPktSize);
	  m_XferSzLabel.SetWindowText(xferSzStr);

	  m_PacketsCombo.SetCurSel(1);
	  m_QueueSzEdit.SetWindowText("4");
    } else
	  m_XferSzLabel.SetWindowText("0");
}


void CStreamerDlg::OnTimer(UINT nIDEvent) 
{
	char szBuf[32];
	char s[24];	ZeroMemory(s, 24);



	if (bCounterInit)
	{
		LONG lCycle = GetCPUCycle();

		wsprintf ( szBuf, "%d%%", lCycle);
		m_CpuValue.SetWindowText(szBuf);
		m_CpuProgress.SetPos(lCycle);

		ulCpuCycles += lCycle;
		ulCpuCount++;
		lCycle = ulCpuCycles / ulCpuCount;
		wsprintf ( szBuf, "%d%%", lCycle);
		m_AvgCpuValue.SetWindowText(szBuf);

		// Process the received data - UI stuff
		ltoa(Successes, s,10);	m_SuccessLabel.SetWindowText(s);
		ltoa(Failures, s,10);	m_FailureLabel.SetWindowText(s);
		ltoa(Rate, s,10);  m_Rate.SetWindowText(s);
		m_SuccessLabel.RedrawWindow();
		m_FailureLabel.RedrawWindow();
		m_Rate.RedrawWindow();

	}


	
	CDialog::OnTimer(nIDEvent);
}

void CStreamerDlg::OnDestroy() 
{
	CDialog::OnDestroy();
	
	if (m_nTimer)
		KillTimer(m_nTimer);   
}

