#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>

#include "efmplus.h"
#include <fec.h>

static int cnt, last;

typedef unsigned long uint32_t;
typedef unsigned long long uint64_t;

		/* EFMplus (simplified) decode table */
#define EFMP_INVALID 0x100
int efmp_decode[65536][2]; // plain | (invalid << 8)

		/* sync codes for acquiring bit-sync */
const uint32_t syncword[2*8*2] =
	{0x12440011, 0x12040011, // SY0
	 0x04040011, 0x04440011, // SY1
	 0x10040011, 0x10440011, // SY2
	 0x08040011, 0x08440011, // SY3
	 0x20040011, 0x20440011, // SY4 
	 0x22440011, 0x22040011, // SY5
	 0x24840011, 0x20840011, // SY6
	 0x24440011, 0x24040011, // SY7
	 0x92040011, 0x92440011, // SY0
	 0x84440011, 0x84040011, // SY1
	 0x90440011, 0x90040011, // SY2
	 0x82440011, 0x82040011, // SY3
	 0x88440011, 0x88040011, // SY4
	 0x89040011, 0x81040011, // SY5
	 0x90840011, 0x80440011, // SY6
	 0x88840011, 0x80840011, // SY7
};


uint32_t last32;

unsigned char current_record_frame[182*13];

		/* scrambling */
int lfsr_val;

int get_lfsr_byte(void)
{
	int i;
	int ret = lfsr_val;
	for (i=0; i<8; ++i)
	{
		int feed = (!(lfsr_val & 0x400)) ^ (!(lfsr_val & 0x4000));
		lfsr_val <<= 1;
		lfsr_val  |= feed;
		lfsr_val  &= 0x7FFF;
	}
	return ret;
}

unsigned char lfsr_table[32767];

void scramble(unsigned char *sector, int n, int offset)
{
	int i;
	for (i=0; i<n; ++i)
		sector[i] ^= lfsr_table[(offset + i) % 32767];
}

int descrambling_mode = 0;
int gamecube_discid_modifier = -1;

		/* EDC, taken from rb's original gc seed calculation tool */
#define SHORTPOLY     0x80000011  // x^32 + x^31 + x^4 + 1

uint32_t edc_table[256];

void precalc_edc_table(void)
{
	int i,j;
	uint32_t entry;

	for(j=0;j<0x100;j++)
	{
		entry=j<<24;
		for(i=0;i<8;i++)
		{
			if(entry&0x80000000) { entry = entry << 1; entry=entry^SHORTPOLY; }
			else entry=entry << 1;
		}
		edc_table[j]=entry;
	}
}

uint32_t calculate_edc(unsigned char *buffer, uint32_t size)
{
	uint32_t idx=0;
	uint64_t edc=0;

	if(size<4) return(0);

		// preload register
	edc = edc | *(buffer++); edc = edc << 8;
	edc = edc | *(buffer++); edc = edc << 8;
	edc = edc | *(buffer++); edc = edc << 8;
	edc = edc | *(buffer++);

		// process all input bytes
	while(idx++<size)
	{
		edc = edc << 8;
		if(idx<size-3) edc = edc | *(buffer++);
		edc = edc ^ edc_table[(edc>>32)&0xFF];
	}

	return( (uint32_t) edc );
}

		/* scramble bruteforce */

int edc_calc_table[32767];

void initialize_scramble_bruteforce(void)
{
	int i;
	for (i=0; i<32767; ++i)
	{
		unsigned char fake_sector[2060];
		memset(fake_sector, 0, 12 + 2048);
		scramble(fake_sector + 12, 2048, i);
			edc_calc_table[i] = calculate_edc(fake_sector, 2060);
	}
}

int edc_table_initialized;


int find_edc_diff(uint32_t diff)
{

	if (!edc_table_initialized)
	{
		printf("init scramble bruteforce table\n");
		initialize_scramble_bruteforce();
		edc_table_initialized = 1;
	}

	int i;
	for (i=0; i<32767; ++i)
		if (edc_calc_table[i] == diff)
			return i;
	return -1;
}

		/* ECC */
void *rs_pi, *rs_po;

unsigned char current_ecc_frame[(192 + 16) * 182];

unsigned char current_sector[2064];

	/* ecc frames */
void handle_valid_ecc_frame(void)
{
	int s;
	for (s = 0; s < 16; ++s)
	{
		int row;
					/* copy together payload */
		unsigned char *d = current_sector;
			for (row = 0; row < 12; ++row)
			memcpy(d + row * 172, current_ecc_frame + (s * 12 + row) * 182, 172);
		int lba = (d[1] << 16) | (d[2] << 8) | d[3];

		int seed_modifier = (lba >> 4) & 0xF; /* should be the same for every of these sectors */
			int seed_offset;
			if (descrambling_mode == 0) // dvd
			seed_offset = seed_modifier * 0x800; /* normal DVDs */
		else if (descrambling_mode == 1) // gamecube
		{
			if (lba == 0x30000)
			{
				int csum = 0, i;
				for (i=0; i<6; ++i)
					csum += current_sector[i + 6];
				csum += csum >> 4;
				csum &= 0xF;
				printf("gamecube gameid checksum: 0x%x\n", csum);
				gamecube_discid_modifier = csum;
			}
					if (lba >= 0x30000 && lba < 0x30010)
			{
				seed_offset = 0x3c00;
			} else if (lba >= 0x30010)
			{
				if (gamecube_discid_modifier == -1)
				{
					printf("0x30000 not yet seen, ignoring sector\n");
					return;
				}
				seed_offset = 0x3c00 + (gamecube_discid_modifier ^ seed_modifier) * 0x800;
			} else
				seed_offset = seed_modifier * 0x800; /* leadin is normally scrambled */
		}

		scramble(current_sector + 12, 2048, seed_offset);
			uint32_t calculated_edc = calculate_edc(current_sector, 2060);
		uint32_t oedc = (current_sector[2060] << 24) | (current_sector[2061] << 16) | (current_sector[2062] << 8) | (current_sector[2063]);
			if (calculated_edc != oedc)
		{
			printf("EDC mismatch! (%08lx cs. %08lx)\n", calculated_edc, oedc);
			printf("trying to find correct scrambling value..\n");
						/* first, remove old (wrong) scrambling. (this could be done more efficient, but makes things more clear) */
			scramble(current_sector + 12, 2048, seed_offset);
			calculated_edc ^= edc_calc_table[seed_offset]; /* also do this for the EDC value */
						/* we can easily find the correct scrambling value because of EDC linearity */
			int correct_scramble_offset = find_edc_diff(calculated_edc ^ oedc);

			if (correct_scramble_offset == -1)
			{
				printf("ERROR: failed to find correct scrambling.\n");
				return;
			}
					seed_offset = correct_scramble_offset; /* use found value */

			printf("scrambling seed corrected to offset=%04x (seed=%04x)\n", seed_offset, lfsr_table[seed_offset]);

			scramble(current_sector + 12, 2048, seed_offset);
						/* just to verify: EDC must match now */
			calculated_edc = calculate_edc(current_sector, 2060);
			if (calculated_edc != oedc)
			{
				printf("ERROR: edc mismatch after correction. this should not happen.\n");
				return;
			}

		} else
			printf("EDC ok!\n");
			printf("writing PSN: %08x, seed: %08x\n", lba, seed_offset);
	//		if (lba >= 0x30000)
			write(2, current_sector, 2064);
	}
}

void handle_ecc_frame(void)
{
	printf("handle ecc frame. LBA numbers:\n");
	int i, lba;
	for (i=0; i<16; ++i)
	{
		unsigned char *d = current_ecc_frame + 12 * i * 182;
		lba = (d[0] << 24) | (d[1] << 16) | (d[2] << 8) | d[3];
		printf("  %08x\n", lba);
	}

#if 0
	int x, y;
	for (y=0; y<(192+16); ++y)
	{
		for (x=0; x<182; ++x)
			printf("%02x ", current_ecc_frame[y * 182 + x]);
		printf("\n");
	}
#endif

	int pitotal = 0, piuncorr = 0, pototal = 0, pouncorr = 0;

		/* handle PI */
	for (i=0; i<192 + 16; ++i)
	{
		unsigned char *d = current_ecc_frame + i * 182;
		int nr_err = decode_rs_char(rs_pi, d, 0, 0);
//		printf("[ECC]  row %d: %d errors\n", i, nr_err);
		if (nr_err >= 0)
			pitotal += nr_err;
		else
			piuncorr++;
	}

		/* handle PO */
	for (i = 0; i < 172; ++i)
	{
		unsigned char col[192 + 16];
		int a;
		for (a = 0; a < 192 + 16; ++a)
			col[a] = current_ecc_frame[i + a * 182];
		int nr_err = decode_rs_char(rs_po, col, 0, 0);

		if (nr_err >= 0)
			pototal += nr_err;
		else
			pouncorr++;

//		printf("[ECC]  col %d: %d errors\n", i, nr_err);
		for (a = 0; a < 192 + 16; ++a)
			current_ecc_frame[i + a * 182] = col[a];
	}
	printf("[ECC] [%08x] PI: %d (%d uncor.), PO: %d (%d uncor.)\n", lba, pitotal, piuncorr, pototal, pouncorr);
	if (!pouncorr)
		handle_valid_ecc_frame();
	else
		printf("[ECC] throwing away uncorrectable frame\n");
}

		/* recording frames */
void *rs_id;

void handle_record_frame(void)
{
	unsigned char *d = current_record_frame;
	unsigned char uncor[6];
	memcpy(uncor, d, 6);
	int lba_uncor = (d[0] << 24) | (d[1] << 16) | (d[2] << 8) | d[3];
	int errors_in_id = decode_rs_char(rs_id, d, 0, 0);
	if (errors_in_id == -1)
	{
		printf("[RF] uncorrectable ID, ignoring recording frame\n");
		return;
	}
	int lba = (d[0] << 24) | (d[1] << 16) | (d[2] << 8) | d[3];
	static int last_lba;
	if (errors_in_id)
	{
		printf("[RF] [ID corrected] ID: %d errors (uncor=%08x, cor=%08x)\n", errors_in_id, lba_uncor, lba);
		printf("[RF] [ID corrected] before: %02x %02x %02x %02x %02x %02x\n", 
			uncor[0], uncor[1], uncor[2], uncor[3], uncor[4], uncor[5]);
		printf("[RF] [ID corrected] after: %02x %02x %02x %02x %02x %02x\n", 
			d[0], d[1], d[2], d[3], d[4], d[5]);
	}
	if (decode_rs_char(rs_id, d, 0, 0))
		lba = last_lba + 1;
	if ((lba & 0xF) && (last_lba + 1) != lba)
		printf("[RF] unexpected lba: last=%08x, this=%08x\n", last_lba, lba);
	last_lba = lba;
	printf("[RF] LBA: %08x\n", lba);
	int index = lba & 0xF;
	memcpy(current_ecc_frame + 12 * 182 * index, d, 12 * 182);
	memcpy(current_ecc_frame + (192 + index) * 182, d + 12 * 182, 182);
	if (index == 0xF)
	{
		handle_ecc_frame();
		memset(current_ecc_frame, 0, (192+16)*182);
	}
}

int state;
int row; // half-row

const int sync_table[26] = {0, 5, 1, 5, 2, 5, 3, 5, 4, 5, 1, 6, 2, 6, 3, 6, 4, 6, 1, 7, 2, 7, 3, 7, 4, 7};

int sync_good;

int get_bit();
unsigned short get_16bit();

int last_sync = -1, sync_state_correct;

#define SYNC_INDEX_TO_SY(x) ((x/2)&7)

void put(int b)
{
		/* NRZI decode */
	int this_bit = b;
	b ^= last;
	last = this_bit;
	cnt++;
		/* serial->parallel */
	last32 <<= 1;
	last32 |= b;
	state++;
	if ((state < 32+4) && (state > 32-4) && ((last32 & 0xFFFF) == 0x0011))
	{
		if (state != 32)
			printf("[BIT] resync from %d\n", state);
		state = 32;
	}

//	printf("s:%d\n", state);
	if (state < 31)
		return;
	else if (state == 32)
	{
//		printf("[BIT] %08x\n", last32);
			int sync_ok = 0;
	//		printf("[BIT] searching for %d %d %d or %d\n", sync_table[row] * 2, sync_table[row] * 2 + 1, sync_table[row] * 2 + 16, sync_table[row] * 2 + 16 + 1);
			if (last32 == syncword[sync_table[row] * 2] || last32 == syncword[sync_table[row] * 2 + 1] || last32 == syncword[sync_table[row] * 2 + 16] || last32 == syncword[sync_table[row] * 2 + 16 + 1])
		{
			sync_ok = 1;
			last_sync = sync_table[row];
			sync_state_correct = 4;
		}
			if (!sync_ok)
		{
			int i;
			for (i=0; i<32; ++i)
				if (syncword[i] == last32)
					break;
			if (i != 32)
			{
				int s = SYNC_INDEX_TO_SY(i);
				printf("[BIT] found unexpected, but valid sync SY%d (last was SY%d), real_row=%d\n", s, last_sync, row);
							if (sync_state_correct)
					sync_state_correct--;
							int nr;
				for (nr = 0; nr < 26; nr += 2)
				{
					if ((sync_table[nr] == last_sync) && (sync_table[(nr+1)%26] == s))
					{
						printf("[BIT] -> row=%d, real_row=%d\n", nr, row);
											if (!sync_state_correct)
						{
							printf("overriding.\n");
							row = nr + 1;
							sync_ok = 1;
						}
					}
				}
				last_sync = s;
			}
		}

		if (sync_ok)
		{
//			printf("[BIT] sync ok (row=%d)\n",row);
			sync_good = 5;
		} else if (sync_good)
		{
			printf("[BIT] (SYNC missing! row=%d, %d) (%08x vs. %08x %08x %08x %08x)\n", row, sync_good,			last32, 
			syncword[sync_table[row] * 2],
			syncword[sync_table[row] * 2 + 1],
			syncword[sync_table[row] * 2 + 16],
			syncword[sync_table[row] * 2 + 16 + 1]);
			sync_good--;
			if (sync_state_correct)
				sync_state_correct--;
		} else
		{
			printf(".");
			sync_state_correct = 0;
			state = 31; /* continue searching for sync */
			return;
		}
	} else if ((state > 32 + 16) && !(state & 0xF)) /* every 16 bits, but not for the first one (we need look-ahead for EFM+ decode) */
	{
		int cw_to_decode = (last32 >> 16) & 0xFFFF;
		int next_state_is_2 = (last32 & 0x8008) == 0;

//		printf("CW:%04x n=%d ", cw_to_decode, next_state_is_2);
			int decode_result = efmp_decode[cw_to_decode][next_state_is_2];
			if (decode_result & EFMP_INVALID)
		{
			printf("[BIT] -> INVALID!\n");
					/* TODO: mark erasure for PI */
		} else
		{
//			printf("[BIT] -> %02x\n", decode_result & 0xFF);
		}
		int ptr = (state - (32 + 32)) / 16;
	//		printf("[BIT] state: %d, ptr: %d, row: %d\n", state, ptr, row);
			current_record_frame[ptr + row * 91] = decode_result & 0xFF;

		if (state == 1456 + 32 + 16)
		{
//			printf("[BIT] current row finished.\n");
			++row;
			state = 16;
					if (row == 26)
			{
//				printf("[BIT] recording frame finished\n");
				handle_record_frame();
				memset(current_record_frame, 0, 182 * 13);
				row = 0;
			}
		}
	}
}


void init_efmplus(void)
{
	int i, s;
	for (i=0; i<65536; ++i)
	{
		efmp_decode[i][0] = EFMP_INVALID;
		efmp_decode[i][1] = EFMP_INVALID;
	}

	for (i=0; i<256 + 88; ++i)
		for (s=0; s<4; ++s)
		{
			int entry = efmp_cw[i][s];
			int ns = efmp_ns[i][s];
			int a = ns == 1; // next_state_is_2
					if ((efmp_decode[entry][a] != EFMP_INVALID) && ((efmp_decode[entry][a]&0xFF) != (i&0xFF)))
			{
				printf("[EFM+] ERROR: %02x:%d is already used (by %02x:%d)\n", i, s, efmp_decode[entry][a] & 0xFF, (efmp_decode[entry][a] >> 8) & 3);
				exit(0);
			}

			efmp_decode[entry][a] = i & 0xFF;

			if ((ns != 1) && (ns != 2))
			{
				if ((efmp_decode[entry][!a] != EFMP_INVALID) && ((efmp_decode[entry][!a]&0xFF) != (i&0xFF)))
				{
					printf("[EFM+] ERROR: alt %02x:%d is already used (by %02x:%d)\n", i, s, efmp_decode[entry][a] & 0xFF, (efmp_decode[entry][a] >> 8) & 3);
					exit(0);
				}
				efmp_decode[entry][!a] = i & 0xFF;
			}

//			printf("[EFM+] %d:%04x:%d = %04x\n", s, entry, a, i | (ns<<8));
		}
}


FILE *input_file;


int main(int argc, char **argv)
{
	printf("init efm decode tables..\n");
	init_efmplus();
	printf("init rs\n");

#define PI_N 182
#define PI_K 172
	rs_pi = init_rs_char(8, 0x11d, 0, 1, PI_N - PI_K, 255 - PI_N);
#define PO_N 208
#define PO_K 192
	rs_po = init_rs_char(8, 0x11d, 0, 1, PO_N - PO_K, 255 - PO_N);
#define ID_N 6
#define ID_K 4
	rs_id = init_rs_char(8, 0x11d, 0, 1, ID_N - ID_K, 255 - ID_N);
	if (!(rs_pi && rs_po && rs_id))
	{
		printf("init_rs_char failed!\n");
		return 1;
	}
	printf("init lfsr\n");
	int i;
	lfsr_val = 1;
	for (i = 0; i < 32767; ++i)
	{
		lfsr_table[i] = get_lfsr_byte();
	}
	printf("init edc\n");
	precalc_edc_table();
	printf("done!\n");
	input_file = stdin;
	if (argc == 2)
	{
		input_file = fopen(argv[1], "rb");
		if (!input_file)
		{
			perror(argv[1]);
			return 1;
		}
	}
//	fseek(input_file, 100*1024*1024, SEEK_SET);

	while (1)
	{
		unsigned char b;
			if (fread(&b, 1, 1, input_file) != 1)
			break;
			int i;
		for (i=7; i>=0; --i)
			if (b & 1<<i)
				put(1);
			else
				put(0);
	}
	return 0;
}
